<?php
/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

namespace Civi\FormProcessor\API;

use Civi\API\Event\ResolveEvent;
use Civi\API\Event\RespondEvent;
use Civi\API\Events;
use Civi\API\Provider\ProviderInterface as API_ProviderInterface;
use Civi\FormProcessor\DataBag;
use Civi\FormProcessor\Utils\Cache;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * This class connects the enabled form processors to
 * the the api interface of CiviCRM. That way the form processors
 * could be executed over the api.
 */
class FormProcessorDefaults extends FormProcessor implements API_ProviderInterface, EventSubscriberInterface {

  public function __construct() {

  }

  public function onApiResolve(ResolveEvent $event) {
    $apiRequest = $event->getApiRequest();
    if (strtolower($apiRequest['entity']) == 'formprocessordefaults') {
      $event->setApiProvider($this);
      if (strtolower($apiRequest['action']) == 'getfields' || strtolower($apiRequest['action']) == 'getoptions') {
        $event->stopPropagation();
      }
    }
  }

  protected function getFields($formProcessorName, $apiParams, $fieldName = NULL, $useCache=true) {
    $fields = [];
    $hash = md5(json_encode($apiParams));
    $cacheKey = 'FormProcessorDefaults.getfields.'.strtolower($formProcessorName).'.'.$hash;
    if ($useCache && ($cache = Cache::get($cacheKey))) {
      return $cache;
    }

    try {
      $formProcessor = self::getFormProcessor($formProcessorName);
    } catch (\API_Exception $e) {
      return;
    }

    // Process all inputs of the formprocessor.
    foreach ($formProcessor['default_data_inputs'] as $input) {
      $field = array(
        'name' => $input['name'],
        'title' => $input['name'],
        'description' => '',
        'type' => $input['type']->getCrmType(),
        'api.required' => $input['is_required'],
        'api.aliases' => [],
        'entity' => 'FormProcessor',
      );
      if ($input['type'] instanceof OptionListInterface) {
        $field['options'] = $input['type']->getOptions($apiParams);
      }
      // Set a default value
      if (isset($input['default_value']) && $input['default_value'] != '') {
        $field['api.default'] = $input['type']->getDefaultValue($input['default_value']);
      }
      if ($input['type']->getCrmType()) {
        $fields[$input['name']] = $field;
      }
    }
    Cache::set($cacheKey, $fields);
    return $fields;
  }

  /**
   * @param array $apiRequest
   *   The full description of the API request.
   *
   * @return array
   *   structured response data (per civicrm_api3_create_success)
   * @throws \Exception
   * @see civicrm_api3_create_success
   */
  public function invoke($apiRequest) {
    switch (strtolower($apiRequest['action'])) {
      case 'getfields':
        // Do get fields
        return $this->invokeGetFields($apiRequest);
        break;
      case 'getoptions':
        // Do get options
        return $this->invokeGetOptions($apiRequest);
        break;
      default:
        return $this->invokeFormProcessorDefaults($apiRequest);
        break;
    }
  }

  /**
   * @param array $apiRequest
   *   The full description of the API request.
   *
   * @return array
   *   structured response data (per civicrm_api3_create_success)
   * @throws \API_Exception
   * @see civicrm_api3_create_success
   */
  public function invokeFormProcessorDefaults($apiRequest) {
    $actionProvider = form_processor_get_action_provider_for_default_data();
    $dataBag = new DataBag();

    $params = $apiRequest['params'];

    $formProcessor = self::getFormProcessor($apiRequest['action']);

    // Validate the parameters.
    foreach ($formProcessor['default_data_inputs'] as $input) {
      $inputType = $input['type'];
      unset($input['type']);
      $objInput = new \CRM_FormProcessor_BAO_FormProcessorDefaultDataInput();
      $objInput->copyValues($input);

      // Set default value
      if (!isset($params[$input['name']]) && isset($input['default_value']) && $input['default_value'] != '') {
        $params[$input['name']] = $input['type']->getDefaultValue($input['default_value']);
      }

      if ($input['is_required'] && !isset($params[$input['name']])) {
        throw new \API_Exception('Parameter ' . $input['name'] . ' is required');
      }
      if (isset($params[$input['name']]) && !empty($params[$input['name']]) && !$inputType->validateValue($params[$input['name']], $params)) {
        throw new \API_Exception('Parameter ' . $input['name'] . ' is invalid');
      }
      // Check the validations on the input.
      if (isset($params[$input['name']]) && $params[$input['name']] != "") {
        foreach ($input['validators'] as $validator) {
          if (!$validator['validator']->validate($params[$input['name']])) {
            throw new \API_Exception($validator['validator']->getInvalidMessage() . ' (Parameter ' . $input['name'] . ')');
          }
        }
      }

      $dataBag->setInputData($objInput, $inputType->normalizeValue($params[$objInput->name]));
    }

    // Execute the actions
    $actionParams = array();
    foreach ($formProcessor['default_data_actions'] as $action) {
      $actionClass = $actionProvider->getActionByName($action['type']);
      $actionClass->getConfiguration()->fromArray($action['configuration']);
      $objAction = new \CRM_FormProcessor_BAO_FormProcessorDefaultDataAction();
      $objAction->copyValues($action);

      // Create a parameter bag for the action
      $mappedParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['mapping'], $actionProvider);

      // Create a condition class for this action
      if (!is_array($action['condition_configuration'])) {
        $action['condition_configuration'] = [];
      }
      if (!isset($action['condition_configuration']['parameter_mapping']) || !is_array($action['condition_configuration']['parameter_mapping'])) {
        $action['condition_configuration']['parameter_mapping'] = [];
      }
      $mappedConditionParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['condition_configuration']['parameter_mapping'], $actionProvider);
      if (!isset($action['condition_configuration']['output_mapping']) || !is_array($action['condition_configuration']['output_mapping'])) {
        $action['condition_configuration']['output_mapping'] = [];
      }
      $mappedConditionOutputParameterBag = $this->convertDataBagToMappedParameterBag($dataBag, $action['condition_configuration']['output_mapping'], $actionProvider);
      $condition = \CRM_FormProcessor_Condition::getConditionClass($action['condition_configuration']);
      $actionClass->setCondition($condition);

      try {
        // Now execute the action
        $outputBag = $actionClass->execute($mappedParameterBag, $mappedConditionParameterBag, $mappedConditionOutputParameterBag);
        // Add the output of the action to the data bag of this action.
        $dataBag->setActionDataFromActionProviderParameterBag($objAction, $outputBag);
      } catch (\Exception $e) {
        throw new Exception($action['title'], $e);
      }
    }

    $return = array();
    foreach ($formProcessor['default_data_output_configuration'] as $field => $alias) {
      foreach ($formProcessor['inputs'] as $input) {
        if ($input['name'] == $field) {
          $return[$field] = $input['type']->denormalizeValue($dataBag->getDataByAlias($alias));
        }
      }
    }
    return $return;
  }

  /**
   * Converts a DataBag object to a mapped parameterBag.
   *
   * @param DataBag $dataBag
   * @param array $mapping
   * @param $actionProvider
   *
   * @return ParameterBag;
   */
  protected function convertDataBagToMappedParameterBag(DataBag $dataBag, $mapping, $actionProvider) {
    $parameterBag = $actionProvider->createParameterBag();
    $fields = $dataBag->getAllAliases();
    foreach ($fields as $field) {
      $parameterBag->setParameter($field, $dataBag->getDataByAlias($field));
    }
    $mappedParameterBag = $actionProvider->createdMappedParameterBag($parameterBag, $mapping);
    return $mappedParameterBag;
  }

  /**
   * @param int $version
   *   API version.
   *
   * @return array<string>
   */
  public function getEntityNames($version) {
    return array(
      'FormProcessorDefaults',
    );
  }

  /**
   * @param int $version
   *   API version.
   * @param string $entity
   *   API entity.
   *
   * @return array<string>
   */
  public function getActionNames($version, $entity) {
    if (strtolower($entity) != 'formprocessordefaults') {
      return array();
    }

    $cachekey = 'formprocessordefaults.getactions';
    if ($cache = Cache::get($cachekey)) {
      return $cache;
    }

    $params['is_active'] = 1;
    $params['enable_default_data'] = 1;
    $form_processors = \CRM_FormProcessor_BAO_FormProcessorInstance::getValues($params);
    $actions[] = 'getfields';
    $actions[] = 'getoptions';
    foreach ($form_processors as $form_processor) {
      $actions[] = $form_processor['name'];
    }
    Cache::set($cachekey, $actions);
    return $actions;
  }

}
