<?php

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */

namespace Civi\FormProcessor\Type;

use Civi\FormProcessor\Config\ConfigurationBag;
use \Civi\FormProcessor\Config\Specification;
use \Civi\FormProcessor\Config\SpecificationBag;
use \Civi\FormProcessor\Type\AbstractType;
use \Civi\FormProcessor\Type\OptionListInterface;

use \CRM_FormProcessor_ExtensionUtil as E;

class OptionGroupType extends AbstractType implements OptionListInterface {

  private $options;

  private $normalizedOptions;

  private $denormalizedOptions;

  private $optionsLoaded = false;

  /**
   * Get the configuration specification
   *
   * @return SpecificationBag
   */
  public function getConfigurationSpecification() {
    $optionsGroups = array();
    $optionGroupsApi = civicrm_api3('OptionGroup', 'get', array('is_active' => 1, 'options' => array('limit' => 0)));
    foreach($optionGroupsApi['values'] as $optionGroup) {
      $optionsGroups[$optionGroup['name']] = $optionGroup['title'];
    }
    return new SpecificationBag(array(
      new Specification('use_label_as_value', 'Boolean', E::ts('Use label as value'), true, 0, null, array(0=>E::ts('No'), 1=>E::ts('Yes'))),
      new Specification('option_group_name', 'Integer', E::ts('Option group'), true, null, null, $optionsGroups, false),
      new Specification('multiple', 'Boolean', E::ts('Multiple'), false, 0, null, array(0=>E::ts('Single value'), 1=>E::ts('Multiple values'))),
    ));
  }

  public function validateValue($value, $allValues=array()) {
    $multiple = $this->configuration->get('multiple') ? true : false;
    $options = $this->getOptions(array());

    // Correct array values when field is not multiple.
    // this could be caused for example by a drupal webform with
    // a checkbox field. The value is submitted as an array, altough it contains
    // only one value.
    if (!$multiple && is_array($value) && count($value) == 1) {
      $value = reset($value);
    }

    if ($multiple && is_array($value)) {
      foreach ($value as $valueItem) {
        if (\CRM_Utils_Type::validate($valueItem, 'String', FALSE) === NULL) {
          return FALSE;
        }
        if (!isset($options[$valueItem])) {
          return FALSE;
        }
      }
    } elseif (!is_array($value) && $value) {
      if (\CRM_Utils_Type::validate($value, 'String', FALSE) === NULL) {
        return FALSE;
      }

      if (!isset($options[$value])) {
        return false;
      }
    } else {
      return false;
    }

    return true;
  }

  /**
   * Returns the type number from CRM_Utils_Type
   */
  public function getCrmType() {
    return \CRM_Utils_Type::T_STRING;
  }

  public function getOptions($params) {
    $this->loadOptions();
    return $this->options;
  }

  /**
   * Normalize the input value.
   *
   * @param $value
   *
   * @return mixed
   */
  public function normalizeValue($value) {
    $this->loadOptions();
    $multiple = $this->configuration->get('multiple') ? true : false;

    // Correct array values when field is not multiple.
    // this could be caused for example by a drupal webform with
    // a checkbox field. The value is submitted as an array, altough it contains
    // only one value.
    if (!$multiple && is_array($value) && count($value) == 1) {
      $value = reset($value);
    }

    if ($multiple && is_array($value)) {
      $return = array();
      foreach($value as $item) {
        $return[] = $this->normalizedOptions[$item];
      }
      return $return;
    } elseif (!is_array($value)) {
      return $this->normalizedOptions[$value];
    } else {
      return null;
    }
  }

  /**
   * Denormalize the input value.
   *
   * @param $value
   *
   * @return mixed
   */
  public function denormalizeValue($value) {
    $this->loadOptions();
    $multiple = $this->configuration->get('multiple') ? true : false;
    if ($multiple && is_array($value)) {
      $return = array();
      foreach($value as $item) {
        $return[] = $this->denormalizedOptions[$item];
      }
      return $return;
    } elseif (!is_array($value)) {
      return $this->denormalizedOptions[$value];
    } else {
      return null;
    }
  }

  private function loadOptions() {
    if ($this->optionsLoaded) {
      return;
    }

    $this->options = array();
    $this->normalizedOptions = array();
    $this->denormalizedOptions = array();

    $use_label = $this->configuration->get('use_label_as_value') ? true : false;
    $option_group_name = $this->configuration->get('option_group_name');
    if (!$option_group_name) {
      return;
    }
    $optionsApi = civicrm_api3('OptionValue', 'get', [
      'option_group_id' => $option_group_name,
      'is_active' => 1,
      'options' => ['limit' => 0]
    ]);
    foreach($optionsApi['values'] as $optionValue) {
      $value = $optionValue['value'];
      if ($use_label) {
        $value = $optionValue['label'];
      }
      $this->options[$value] = $optionValue['label'];
      $this->normalizedOptions[$value] = $optionValue['value'];
      $this->denormalizedOptions[$optionValue['value']] = $value;
    }

    $this->optionsLoaded = true;
  }

  public function setConfiguration(ConfigurationBag $configuration) {
    parent::setConfiguration($configuration);
    $this->normalizedOptions = null;
    $this->denormalizedOptions = null;
    $this->options = null;
    return $this;
  }

  /**
   * Sets the default values of this action
   */
  public function setDefaults() {
    parent::setDefaults();
    $this->normalizedOptions = null;
    $this->denormalizedOptions = null;
    $this->options = null;
    $this->optionsLoaded = false;
  }

}
